"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getTypescriptPropsTypeName_1 = require("../../domain/getTypescriptPropsTypeName");
var uncapitalizeFirstLetter_1 = require("../../utils/uncapitalizeFirstLetter");
var getGraphqlTypeString_1 = require("../../domain/getGraphqlTypeString");
var generatePropsType_1 = require("../helper/generatePropsType");
var buildVariablesDeclarationString_1 = require("../../domain/buildVariablesDeclarationString");
var buildVariablesPassString_1 = require("../../domain/buildVariablesPassString");
var getTypescriptTypeString_1 = require("../../domain/getTypescriptTypeString");
var generateResultTypeFields_1 = require("../helper/generateResultTypeFields");
var renderQuery_1 = require("../render/renderQuery");
var renderMethod_1 = require("../render/renderMethod");
var renderFragment_1 = require("../render/renderFragment");
var renderApolloCall_1 = require("../render/renderApolloCall");
var rootType_1 = require("../../utils/rootType");
function default_1(field, types) {
    var queryName = field.name;
    var propsType = getTypescriptPropsTypeName_1.default('Query', queryName);
    var inputs = field.args || [];
    var methodName = uncapitalizeFirstLetter_1.default(queryName);
    var hasInputs = !!inputs.length;
    var returnClassFullname = getTypescriptTypeString_1.default({
        type: field.type,
    });
    var returnClassName = getGraphqlTypeString_1.default({
        type: field.type,
        capitalizeName: true,
        onlyName: true,
    });
    var returnGraphqlTypeName = returnClassName;
    var fragmentName = returnClassName + 'Data';
    var props = generatePropsType_1.default(propsType, inputs);
    var resultTypeFields = generateResultTypeFields_1.default(field.type, types);
    var hasResultType = !!resultTypeFields;
    var variablesDeclarationString = buildVariablesDeclarationString_1.default(inputs);
    var variablesString = buildVariablesPassString_1.default(inputs);
    var type = types.find(function (x) { return x.name === returnGraphqlTypeName; });
    var method = renderMethod_1.default({
        methodName: methodName,
        hasProps: hasInputs,
        propsType: propsType,
        renderContent: function () {
            // Render Query
            return renderFragment_1.default(type) +
                renderQuery_1.default({
                    fragmentName: fragmentName,
                    graphqlTypeName: returnGraphqlTypeName,
                    hasFragment: hasResultType,
                    queryName: queryName,
                    variablesDeclarationString: variablesDeclarationString,
                    variablesString: variablesString,
                }) +
                renderApolloCall_1.default({
                    rootType: rootType_1.RootType.Query,
                    hasVariables: hasInputs,
                    returnType: returnClassFullname,
                });
        },
    });
    return {
        method: method,
        props: props,
    };
}
exports.default = default_1;
