"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var generateRootType_1 = require("./generateRootType");
var rootType_1 = require("../utils/rootType");
var generateType_1 = require("./helper/generateType");
var renderProxy_1 = require("./render/renderProxy");
function default_1(introspectionSchema) {
    var queryTypeName = introspectionSchema.queryType.name, mutationTypeName = introspectionSchema.mutationType.name, subscriptionTypeName = introspectionSchema.subscriptionType.name, types = introspectionSchema.types;
    var queryType = types.find(function (x) { return x.name === queryTypeName; });
    var mutationType = types.find(function (x) { return x.name === mutationTypeName; });
    var subscriptionType = types.find(function (x) { return x.name === subscriptionTypeName; });
    var otherTypes = types.filter(function (x) {
        return (x !== queryType) &&
            (x !== mutationType) &&
            (x !== subscriptionType);
    });
    // start generation
    var generatedQuery = generateRootType_1.default(rootType_1.RootType.Query, otherTypes)(queryType);
    var generatedMutation = generateRootType_1.default(rootType_1.RootType.Mutation, otherTypes)(mutationType);
    var generatedSubscription = generateRootType_1.default(rootType_1.RootType.Subscription, otherTypes)(subscriptionType);
    var generatedOtherTypes = otherTypes
        .sort(sortTypesByKind)
        .map(generateType_1.default)
        .filter(function (x) { return !!x; })
        .join('\n');
    return renderProxy_1.default({
        generatedQuery: generatedQuery,
        generatedMutation: generatedMutation,
        generatedSubscription: generatedSubscription,
        generatedOtherTypes: generatedOtherTypes,
    });
}
exports.default = default_1;
function sortTypesByKind(a, b) {
    return getSortValue(b) - getSortValue(a);
}
function getSortValue(x) {
    switch (x.kind) {
        case 'SCALAR':
            return 10;
        case 'UNION':
            return 9;
        case 'ENUM':
            return 8;
        case 'INPUT_OBJECT':
            return 7;
        case 'OBJECT':
            return 6;
        default:
            return 0;
    }
}
