"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getTypescriptField_1 = require("./getTypescriptField");
function default_1(type) {
    if (type.name.startsWith('__')) {
        return;
    }
    switch (type.kind) {
        case 'OBJECT':
            return objectType(type);
        case 'INPUT_OBJECT':
            return inputObjectType(type);
        case 'ENUM':
            return enumType(type);
        case 'SCALAR':
            return scalarType(type);
        case 'UNION':
            return unionType(type);
        default:
            console.log('MISSED GENERATION FOR', type.kind, type.name);
            return;
    }
}
exports.default = default_1;
function objectType(type) {
    var fields = type.fields
        .map(function (x) { return getTypescriptField_1.default(x.name, x.type, { isNull: false, isList: false }); })
        .map(function (x) { return "\t" + x; })
        .join('\n');
    var typeName = type.name;
    return "\ninterface " + typeName + " {\n" + fields + "\n}";
}
function inputObjectType(type) {
    var fields = type.inputFields
        .map(function (x) { return getTypescriptField_1.default(x.name, x.type, { isNull: false, isList: false }); })
        .map(function (x) { return "\t" + x; })
        .join('\n');
    var typeName = type.name;
    return "\ninterface " + typeName + " {\n" + fields + "\n}";
}
function enumType(type) {
    var typeName = type.name;
    var fields = type.enumValues
        .map(function (x) { return "\t" + x.name + " = '" + x.name + "',"; })
        .join('\n');
    return "\nenum " + typeName + " {\n" + fields + "\n}";
}
function scalarType(type) {
    var typeName = type.name;
    var tsType = '';
    switch (typeName) {
        case 'ID':
            tsType = 'any';
            break;
        case 'String':
            tsType = 'string';
            break;
        case 'Int':
            tsType = 'number';
            break;
        case 'Boolean':
            tsType = 'boolean';
            break;
        case 'DateTime':
        case 'Time':
            tsType = 'Date';
            break;
        // Date already exists in JS
        case 'Date':
            return '';
        case 'JSON':
            tsType = 'any';
            break;
        default:
            console.log('MISSED SCALAR TYPE', typeName);
            return '';
    }
    return "type " + typeName + " = " + tsType;
}
function unionType(type) {
    var typeName = type.name;
    var valueTypes = type.possibleTypes
        .map(function (x) { return x.name; }).join(' | ');
    return "type " + typeName + " = " + valueTypes;
}
